@@ -987,53 +987,83 @@ gst_element_get_compatible_pad (GstElement * element, GstPad * pad,
  *
  * Returns: a string with the name of the state.
  */
-const gchar *
+G_CONST_RETURN gchar *
 gst_element_state_get_name (GstState state)
 {
   switch (state) {
 #ifdef GST_DEBUG_COLOR
     case GST_STATE_VOID_PENDING:
       return "VOID_PENDING";
-      break;
     case GST_STATE_NULL:
       return "\033[01;34mNULL\033[00m";
-      break;
     case GST_STATE_READY:
       return "\033[01;31mREADY\033[00m";
-      break;
     case GST_STATE_PLAYING:
       return "\033[01;32mPLAYING\033[00m";
-      break;
     case GST_STATE_PAUSED:
       return "\033[01;33mPAUSED\033[00m";
-      break;
     default:
       /* This is a memory leak */
       return g_strdup_printf ("\033[01;35;41mUNKNOWN!\033[00m(%d)", state);
 #else
     case GST_STATE_VOID_PENDING:
       return "VOID_PENDING";
-      break;
     case GST_STATE_NULL:
       return "NULL";
-      break;
     case GST_STATE_READY:
       return "READY";
-      break;
     case GST_STATE_PLAYING:
       return "PLAYING";
-      break;
     case GST_STATE_PAUSED:
       return "PAUSED";
-      break;
     default:
       /* This is a memory leak */
       return g_strdup_printf ("UNKNOWN!(%d)", state);
 #endif
   }
-  return "";
 }
 
+/**
+ * gst_element_state_change_return_get_name:
+ * @state_ret: a #GstStateChangeReturn to get the name of.
+ *
+ * Gets a string representing the given state change result.
+ *
+ * Returns: a string with the name of the state change result.
+ */
+G_CONST_RETURN gchar *
+gst_element_state_change_return_get_name (GstStateChangeReturn state_ret)
+{
+  switch (state_ret) {
+#ifdef GST_DEBUG_COLOR
+    case GST_STATE_CHANGE_FAILURE:
+      return "\033[01;31mFAILURE\033[00m";
+    case GST_STATE_CHANGE_SUCCESS:
+      return "\033[01;32mSUCCESS\033[00m";
+    case GST_STATE_CHANGE_ASYNC:
+      return "\033[01;33mASYNC\033[00m";
+    case GST_STATE_CHANGE_NO_PREROLL:
+      return "\033[01;34mNO_PREROLL\033[00m";
+    default:
+      /* This is a memory leak */
+      return g_strdup_printf ("\033[01;35;41mUNKNOWN!\033[00m(%d)", state_ret);
+#else
+    case GST_STATE_CHANGE_FAILURE:
+      return "FAILURE";
+    case GST_STATE_CHANGE_SUCCESS:
+      return "SUCCESS";
+    case GST_STATE_CHANGE_ASYNC:
+      return "ASYNC";
+    case GST_STATE_CHANGE_NO_PREROLL:
+      return "NO PREROLL";
+    default:
+      /* This is a memory leak */
+      return g_strdup_printf ("UNKNOWN!(%d)", state_ret);
+#endif
+  }
+}
+
+
 /**
  * gst_element_factory_can_src_caps :
  * @factory: factory to query
@@ -1214,6 +1244,17 @@ prepare_link_maybe_ghosting (GstPad ** src, GstPad ** sink,
   e1 = GST_OBJECT_PARENT (*src);
   e2 = GST_OBJECT_PARENT (*sink);
 
+  if (G_UNLIKELY (e1 == NULL)) {
+    GST_WARNING ("Trying to ghost a pad that doesn't have a parent: %"
+        GST_PTR_FORMAT, *src);
+    return FALSE;
+  }
+  if (G_UNLIKELY (e2 == NULL)) {
+    GST_WARNING ("Trying to ghost a pad that doesn't have a parent: %"
+        GST_PTR_FORMAT, *sink);
+    return FALSE;
+  }
+
   if (GST_OBJECT_PARENT (e1) == GST_OBJECT_PARENT (e2)) {
     GST_CAT_INFO (GST_CAT_PADS, "%s and %s in same bin, no need for ghost pads",
         GST_OBJECT_NAME (e1), GST_OBJECT_NAME (e2));
@@ -1226,9 +1267,8 @@ prepare_link_maybe_ghosting (GstPad ** src, GstPad ** sink,
   /* we need to setup some ghost pads */
   root = find_common_root (e1, e2);
   if (!root) {
-    g_warning
-        ("Trying to connect elements that don't share a common ancestor: %s and %s\n",
-        GST_ELEMENT_NAME (e1), GST_ELEMENT_NAME (e2));
+    g_warning ("Trying to connect elements that don't share a common "
+        "ancestor: %s and %s", GST_ELEMENT_NAME (e1), GST_ELEMENT_NAME (e2));
     return FALSE;
   }
 
@@ -1934,13 +1974,20 @@ gst_element_query_convert (GstElement * element, GstFormat src_format,
  * Simple API to perform a seek on the given element, meaning it just seeks
  * to the given position relative to the start of the stream. For more complex
  * operations like segment seeks (e.g. for looping) or changing the playback
- * rate or seeking relative to the current position or seeking relative to
- * the end of the stream you should use gst_element_seek ().
+ * rate or seeking relative to the last configured playback segment you should
+ * use gst_element_seek().
  *
- * Note that seeking is usually only possible in PAUSED or PLAYING state.
+ * In a completely prerolled PAUSED or PLAYING pipeline, seeking is always
+ * guaranteed to return %TRUE on a seekable media type or %FALSE when the media
+ * type is certainly not seekable (such as a live stream).
  *
- * Returns: TRUE if the seek operation succeeded (the seek
- *          might not always be executed instantly though)
+ * Some elements allow for seeking in the READY state, in this
+ * case they will store the seek event and execute it when they are put to
+ * PAUSED. If the element supports seek in READY, it will always return %TRUE when
+ * it receives the event in the READY state.
+ *
+ * Returns: %TRUE if the seek operation succeeded (the seek might not always be
+ * executed instantly though)
  *
  * Since: 0.10.7
  */
