@@ -234,7 +234,7 @@ gst_controlled_property_set_interpolation_mode (GstControlledProperty * self,
         self->get_value_array = NULL;
     }
     if (!self->get) {           /* || !self->get_value_array) */
-      GST_WARNING ("incomplete implementation for type %d/%d:'%s'/'%s'",
+      GST_WARNING ("incomplete implementation for type %lu/%lu:'%s'/'%s'",
           self->type, self->base,
           g_type_name (self->type), g_type_name (self->base));
       res = FALSE;
@@ -367,7 +367,7 @@ gst_controlled_property_new (GObject * object, const gchar * name)
           GST_WARNING ("incomplete implementation for paramspec type '%s'",
               G_PARAM_SPEC_TYPE_NAME (pspec));
       }
-      /* TODO what about adding a timedval with timestamp=0 and value=default
+      /* TODO what about adding a timed-val with timestamp=0 and value=default
        * a bit easier for interpolators, example:
        * first timestamp is at 5
        * requested value if for timestamp=3
@@ -448,13 +448,13 @@ gst_controller_find_controlled_property (GstController * self,
  * Creates a new GstController for the given object's properties
  *
  * Returns: the new controller.
- * Since: 0.9
  */
 GstController *
 gst_controller_new_valist (GObject * object, va_list var_args)
 {
   GstController *self;
   GstControlledProperty *prop;
+  gboolean ref_existing = TRUE;
   gchar *name;
 
   g_return_val_if_fail (G_IS_OBJECT (object), NULL);
@@ -482,16 +482,26 @@ gst_controller_new_valist (GObject * object, va_list var_args)
         /* if we don't have a controller object yet, now is the time to create one */
         if (!self) {
           self = g_object_new (GST_TYPE_CONTROLLER, NULL);
-          self->object = object;
+          self->object = g_object_ref (object);
           /* store the controller */
           g_object_set_qdata (object, __gst_controller_key, self);
+          ref_existing = FALSE;
         } else {
-          GST_INFO ("returning existing controller");
+          /* only want one single _ref(), even for multiple properties */
+          if (ref_existing) {
+            g_object_ref (self);
+            ref_existing = FALSE;
+            GST_INFO ("returning existing controller");
+          }
         }
         self->properties = g_list_prepend (self->properties, prop);
       }
     } else {
       GST_WARNING ("trying to control property again");
+      if (ref_existing) {
+        g_object_ref (self);
+        ref_existing = FALSE;
+      }
     }
   }
   va_end (var_args);
@@ -509,13 +519,13 @@ gst_controller_new_valist (GObject * object, va_list var_args)
  * Creates a new GstController for the given object's properties
  *
  * Returns: the new controller.
- * Since: 0.9
  */
 GstController *
 gst_controller_new_list (GObject * object, GList * list)
 {
   GstController *self;
   GstControlledProperty *prop;
+  gboolean ref_existing = TRUE;
   gchar *name;
   GList *node;
 
@@ -534,16 +544,26 @@ gst_controller_new_list (GObject * object, GList * list)
         /* if we don't have a controller object yet, now is the time to create one */
         if (!self) {
           self = g_object_new (GST_TYPE_CONTROLLER, NULL);
-          self->object = object;
+          self->object = g_object_ref (object);
           /* store the controller */
           g_object_set_qdata (object, __gst_controller_key, self);
+          ref_existing = FALSE;
         } else {
-          GST_INFO ("returning existing controller");
+          /* only want one single _ref(), even for multiple properties */
+          if (ref_existing) {
+            g_object_ref (self);
+            ref_existing = FALSE;
+            GST_INFO ("returning existing controller");
+          }
         }
         self->properties = g_list_prepend (self->properties, prop);
       }
     } else {
       GST_WARNING ("trying to control property again");
+      if (ref_existing) {
+        g_object_ref (self);
+        ref_existing = FALSE;
+      }
     }
   }
 
@@ -560,7 +580,6 @@ gst_controller_new_list (GObject * object, GList * list)
  * Creates a new GstController for the given object's properties
  *
  * Returns: the new controller.
- * Since: 0.9
  */
 GstController *
 gst_controller_new (GObject * object, ...)
@@ -585,7 +604,6 @@ gst_controller_new (GObject * object, ...)
  * Removes the given object properties from the controller
  *
  * Returns: %FALSE if one of the given property isn't handled by the controller, %TRUE otherwise
- * Since: 0.9
  */
 gboolean
 gst_controller_remove_properties_valist (GstController * self, va_list var_args)
@@ -620,7 +638,6 @@ gst_controller_remove_properties_valist (GstController * self, va_list var_args)
  * Removes the given object properties from the controller
  *
  * Returns: %FALSE if one of the given property isn't handled by the controller, %TRUE otherwise
- * Since: 0.9
  */
 gboolean
 gst_controller_remove_properties_list (GstController * self, GList * list)
@@ -658,7 +675,6 @@ gst_controller_remove_properties_list (GstController * self, GList * list)
  * Removes the given object properties from the controller
  *
  * Returns: %FALSE if one of the given property isn't handled by the controller, %TRUE otherwise
- * Since: 0.9
  */
 gboolean
 gst_controller_remove_properties (GstController * self, ...)
@@ -685,7 +701,6 @@ gst_controller_remove_properties (GstController * self, ...)
  * Set the value of given controller-handled property at a certain time.
  *
  * Returns: FALSE if the values couldn't be set (ex : properties not handled by controller), TRUE otherwise
- * Since: 0.9
  */
 gboolean
 gst_controller_set (GstController * self, gchar * property_name,
@@ -738,7 +753,6 @@ gst_controller_set (GstController * self, gchar * property_name,
  * Sets multiple timed values at once.
  *
  * Returns: %FALSE if the values couldn't be set (ex : properties not handled by controller), %TRUE otherwise
- * Since: 0.9
  */
 
 gboolean
@@ -758,11 +772,15 @@ gst_controller_set_from_list (GstController * self, gchar * property_name,
     for (node = timedvalues; node; node = g_slist_next (node)) {
       tv = node->data;
       if (G_VALUE_TYPE (&tv->value) == prop->type) {
-        g_return_val_if_fail (GST_CLOCK_TIME_IS_VALID (tv->timestamp), FALSE);
-        /* TODO copy the timed value or just link in? */
-        prop->values =
-            g_list_insert_sorted (prop->values, tv, gst_timed_value_compare);
-        res = TRUE;
+        if (GST_CLOCK_TIME_IS_VALID (tv->timestamp)) {
+          /* TODO copy the timed value or just link in? */
+          prop->values =
+              g_list_insert_sorted (prop->values, tv, gst_timed_value_compare);
+          res = TRUE;
+        } else {
+          g_warning ("GstTimedValued with invalid timestamp passed to %s "
+              "for property '%s'", GST_FUNCTION, property_name);
+        }
       } else {
         GST_WARNING ("incompatible value type for property '%s'", prop->name);
       }
@@ -783,7 +801,6 @@ gst_controller_set_from_list (GstController * self, gchar * property_name,
  * time.
  *
  * Returns: %FALSE if the values couldn't be unset (ex : properties not handled by controller), %TRUE otherwise
- * Since: 0.9
  */
 gboolean
 gst_controller_unset (GstController * self, gchar * property_name,
@@ -856,7 +873,6 @@ gst_controller_unset_all (GstController * self, gchar * property_name)
  * time.
  *
  * Returns: the GValue of the property at the given time, or %NULL if the property isn't handled by the controller
- * Since: 0.9
  */
 GValue *
 gst_controller_get (GstController * self, gchar * property_name,
@@ -888,7 +904,6 @@ gst_controller_get (GstController * self, gchar * property_name,
  * Free the list after done with it.
  *
  * Returns: a copy of the list, or %NULL if the property isn't handled by the controller
- * Since: 0.9
  */
 const GList *
 gst_controller_get_all (GstController * self, gchar * property_name)
@@ -918,7 +933,6 @@ gst_controller_get_all (GstController * self, gchar * property_name)
  *
  * Returns: %TRUE if the controller values could be applied to the object
  * properties, %FALSE otherwise
- * Since: 0.9
  */
 gboolean
 gst_controller_sync_values (GstController * self, GstClockTime timestamp)
@@ -989,7 +1003,6 @@ gst_controller_sync_values (GstController * self, GstClockTime timestamp)
  * The type of the values in the array are the same as the property's type.
  *
  * Returns: %TRUE if the given array(s) could be filled, %FALSE otherwise
- * Since: 0.9
  */
 gboolean
 gst_controller_get_value_arrays (GstController * self,
@@ -1021,7 +1034,6 @@ gst_controller_get_value_arrays (GstController * self,
  * The type of the values in the array are the same as the property's type.
  *
  * Returns: %TRUE if the given array(s) could be filled, %FALSE otherwise
- * Since: 0.9
  */
 gboolean
 gst_controller_get_value_array (GstController * self, GstClockTime timestamp,
@@ -1064,7 +1076,6 @@ gst_controller_get_value_array (GstController * self, GstClockTime timestamp,
  * Sets the given interpolation mode on the given property.
  *
  * Returns: %TRUE if the property is handled by the controller, %FALSE otherwise
- * Since: 0.9
  */
 gboolean
 gst_controller_set_interpolation_mode (GstController * self,
@@ -1169,26 +1180,44 @@ _gst_controller_set_property (GObject * object, guint property_id,
   }
 }
 
+static void
+_gst_controller_dispose (GObject * object)
+{
+  GstController *self = GST_CONTROLLER (object);
+
+  if (self->object != NULL) {
+    g_mutex_lock (self->lock);
+    /* free list of properties */
+    if (self->properties) {
+      GList *node;
+
+      for (node = self->properties; node; node = g_list_next (node)) {
+        GstControlledProperty *prop = node->data;
+
+        g_signal_handler_disconnect (self->object, prop->notify_handler_id);
+        gst_controlled_property_free (prop);
+      }
+      g_list_free (self->properties);
+      self->properties = NULL;
+    }
+
+    /* remove controller from object's qdata list */
+    g_object_set_qdata (self->object, __gst_controller_key, NULL);
+    g_object_unref (self->object);
+    self->object = NULL;
+    g_mutex_unlock (self->lock);
+  }
+
+  if (G_OBJECT_CLASS (parent_class)->dispose)
+    (G_OBJECT_CLASS (parent_class)->dispose) (object);
+}
+
 static void
 _gst_controller_finalize (GObject * object)
 {
   GstController *self = GST_CONTROLLER (object);
-  GList *node;
-  GstControlledProperty *prop;
 
-  /* free list of properties */
-  if (self->properties) {
-    for (node = self->properties; node; node = g_list_next (node)) {
-      prop = node->data;
-      g_signal_handler_disconnect (self->object, prop->notify_handler_id);
-      gst_controlled_property_free (prop);
-    }
-    g_list_free (self->properties);
-    self->properties = NULL;
-  }
   g_mutex_free (self->lock);
-  /* remove controller from objects qdata list */
-  g_object_set_qdata (self->object, __gst_controller_key, NULL);
 
   if (G_OBJECT_CLASS (parent_class)->finalize)
     (G_OBJECT_CLASS (parent_class)->finalize) (object);
@@ -1215,6 +1244,7 @@ _gst_controller_class_init (GstControllerClass * klass)
 
   gobject_class->set_property = _gst_controller_set_property;
   gobject_class->get_property = _gst_controller_get_property;
+  gobject_class->dispose = _gst_controller_dispose;
   gobject_class->finalize = _gst_controller_finalize;
 
   __gst_controller_key = g_quark_from_string ("gst::controller");
