@@ -21,8 +21,13 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
+#include <gst/controller/gstcontroller.h>
+
 #include "tools.h"
-#include <gst/control/control.h>
 
 #include <string.h>
 #include <locale.h>
@@ -48,7 +53,7 @@ n_print (const char *format, ...)
 }
 
 static gboolean
-print_field (GQuark field, GValue * value, gpointer pfx)
+print_field (GQuark field, const GValue * value, gpointer pfx)
 {
   gchar *str = gst_value_serialize (value);
 
@@ -81,6 +86,7 @@ print_caps (const GstCaps * caps, const gchar * pfx)
   }
 }
 
+#if 0
 static void
 print_formats (const GstFormat * formats)
 {
@@ -97,6 +103,7 @@ print_formats (const GstFormat * formats)
     formats++;
   }
 }
+#endif
 
 static void
 print_query_types (const GstQueryType * types)
@@ -116,6 +123,7 @@ print_query_types (const GstQueryType * types)
 }
 
 #ifndef GST_DISABLE_ENUMTYPES
+#if 0
 static void
 print_event_masks (const GstEventMask * masks)
 {
@@ -163,6 +171,7 @@ print_event_masks (const GstEventMask * masks)
     masks++;
   }
 }
+#endif
 #else
 static void
 print_event_masks (const GstEventMask * masks)
@@ -237,13 +246,14 @@ print_interfaces (GType type)
 
   if (ifaces) {
     if (n_ifaces) {
-      g_print ("%sImplemented Interfaces:\n", (_name) ? _name : "");
+      g_print ("%s", _name);
+      g_print (_("Implemented Interfaces:\n"));
       iface = ifaces;
       while (*iface) {
-        g_print ("%s  %s\n", (_name) ? _name : "", g_type_name (*iface));
+        g_print ("%s  %s\n", _name, g_type_name (*iface));
         iface++;
       }
-      g_print ("%s\n", (_name) ? _name : "");
+      g_print ("%s\n", _name);
       g_free (ifaces);
     }
   }
@@ -255,6 +265,7 @@ print_element_properties_info (GstElement * element)
   GParamSpec **property_specs;
   guint num_properties, i;
   gboolean readable;
+  gboolean first_flag;
 
   property_specs = g_object_class_list_properties
       (G_OBJECT_GET_CLASS (element), &num_properties);
@@ -268,26 +279,48 @@ print_element_properties_info (GstElement * element)
     readable = FALSE;
 
     g_value_init (&value, param->value_type);
-    if (param->flags & G_PARAM_READABLE) {
-      g_object_get_property (G_OBJECT (element), param->name, &value);
-      readable = TRUE;
-    }
 
     n_print ("  %-20s: %s\n", g_param_spec_get_name (param),
         g_param_spec_get_blurb (param));
 
+    first_flag = TRUE;
+    n_print ("%-23.23s flags: ", "");
+    if (param->flags & G_PARAM_READABLE) {
+      g_object_get_property (G_OBJECT (element), param->name, &value);
+      readable = TRUE;
+      g_print ((first_flag ? "" : ", "));
+      g_print (_("readable"));
+      first_flag = FALSE;
+    }
+    if (param->flags & G_PARAM_WRITABLE) {
+      g_print ((first_flag ? "" : ", "));
+      g_print (_("writable"));
+      first_flag = FALSE;
+    }
+    if (param->flags & GST_PARAM_CONTROLLABLE) {
+      g_print ((first_flag ? "" : ", "));
+      g_print (_("controllable"));
+      first_flag = FALSE;
+    }
+    n_print ("\n");
+
     switch (G_VALUE_TYPE (&value)) {
-      case G_TYPE_STRING:{
+      case G_TYPE_STRING:
+      {
         GParamSpecString *pstring = G_PARAM_SPEC_STRING (param);
 
         n_print ("%-23.23s String. ", "");
-        g_print ("Default: \"%s\" ",
-            pstring->default_value ? pstring->default_value : "");
+
+        if (pstring->default_value == NULL)
+          g_print ("Default: null ");
+        else
+          g_print ("Default: \"%s\" ", pstring->default_value);
+
         if (readable) {
           const char *string_val = g_value_get_string (&value);
 
           if (string_val == NULL)
-            g_print ("Current: \"\"");
+            g_print ("Current: null");
           else
             g_print ("Current: \"%s\"", string_val);
         }
@@ -395,9 +428,6 @@ print_element_properties_info (GstElement * element)
         break;
       }
       default:
-        if (param->value_type == GST_TYPE_URI) {
-          n_print ("%-23.23s URI", "");
-        }
         if (param->value_type == GST_TYPE_CAPS) {
           const GstCaps *caps = gst_value_get_caps (&value);
 
@@ -420,14 +450,15 @@ print_element_properties_info (GstElement * element)
             j++;
           }
 
-          n_print ("%-23.23s Enum \"%s\" (Current %d, \"%s\")", "",
+          n_print ("%-23.23s Enum \"%s\" Current: %d, \"%s\"", "",
               g_type_name (G_VALUE_TYPE (&value)),
               enum_value, values[j].value_nick);
 
           j = 0;
           while (values[j].value_name) {
-            g_print ("\n%s%-23.23s    (%d): \t%s", "",
-                _name, values[j].value, values[j].value_nick);
+            g_print ("\n%s%-23.23s    (%d): %-16s - %s", "",
+                _name, values[j].value, values[j].value_nick,
+                values[j].value_name);
             j++;
           }
           /* g_type_class_unref (ec); */
@@ -451,14 +482,15 @@ print_element_properties_info (GstElement * element)
             j++;
           }
 
-          n_print ("%-23.23s Flags \"%s\" (Current: %d, \"%s\")", "",
+          n_print ("%-23.23s Flags \"%s\" Current: %d, \"%s\"", "",
               g_type_name (G_VALUE_TYPE (&value)),
               flags_value, (flags ? flags->str : "(none)"));
 
           j = 0;
           while (values[j].value_name) {
-            g_print ("\n%s%-23.23s    (%d): \t%s", "",
-                _name, values[j].value, values[j].value_nick);
+            g_print ("\n%s%-23.23s    (0x%08x): %-16s - %s", "",
+                _name, values[j].value, values[j].value_nick,
+                values[j].value_name);
             j++;
           }
 
@@ -467,6 +499,18 @@ print_element_properties_info (GstElement * element)
         } else if (G_IS_PARAM_SPEC_OBJECT (param)) {
           n_print ("%-23.23s Object of type \"%s\"", "",
               g_type_name (param->value_type));
+        } else if (G_IS_PARAM_SPEC_BOXED (param)) {
+          n_print ("%-23.23s Boxed pointer of type \"%s\"", "",
+              g_type_name (param->value_type));
+        } else if (G_IS_PARAM_SPEC_POINTER (param)) {
+          if (param->value_type != G_TYPE_POINTER) {
+            n_print ("%-23.23s Pointer of type \"%s\".", "",
+                g_type_name (param->value_type));
+          } else {
+            n_print ("%-23.23s Pointer.", "");
+          }
+        } else if (param->value_type == G_TYPE_VALUE_ARRAY) {
+          n_print ("%-23.23s Array of GValues", "");
         } else {
           n_print ("%-23.23s Unknown type %ld \"%s\"", "", param->value_type,
               g_type_name (param->value_type));
@@ -480,6 +524,7 @@ print_element_properties_info (GstElement * element)
   }
   if (num_properties == 0)
     n_print ("  none\n");
+
   g_free (property_specs);
 }
 
@@ -488,7 +533,7 @@ print_pad_templates_info (GstElement * element, GstElementFactory * factory)
 {
   GstElementClass *gstelement_class;
   const GList *pads;
-  GstPadTemplate *padtemplate;
+  GstStaticPadTemplate *padtemplate;
 
   n_print ("Pad Templates:\n");
   if (!factory->numpadtemplates) {
@@ -498,9 +543,9 @@ print_pad_templates_info (GstElement * element, GstElementFactory * factory)
 
   gstelement_class = GST_ELEMENT_CLASS (G_OBJECT_GET_CLASS (element));
 
-  pads = factory->padtemplates;
+  pads = factory->staticpadtemplates;
   while (pads) {
-    padtemplate = (GstPadTemplate *) (pads->data);
+    padtemplate = (GstStaticPadTemplate *) (pads->data);
     pads = g_list_next (pads);
 
     if (padtemplate->direction == GST_PAD_SRC)
@@ -521,9 +566,9 @@ print_pad_templates_info (GstElement * element, GstElementFactory * factory)
     } else
       n_print ("    Availability: UNKNOWN!!!\n");
 
-    if (padtemplate->caps) {
+    if (padtemplate->static_caps.string) {
       n_print ("    Capabilities:\n");
-      print_caps (padtemplate->caps, "      ");
+      print_caps (gst_static_caps_get (&padtemplate->static_caps), "      ");
     }
 
     n_print ("\n");
@@ -538,40 +583,12 @@ print_element_flag_info (GstElement * element)
   n_print ("\n");
   n_print ("Element Flags:\n");
 
-  if (GST_FLAG_IS_SET (element, GST_ELEMENT_COMPLEX)) {
-    n_print ("  GST_ELEMENT_COMPLEX\n");
-    have_flags = TRUE;
-  }
-  if (GST_FLAG_IS_SET (element, GST_ELEMENT_DECOUPLED)) {
-    n_print ("  GST_ELEMENT_DECOUPLED\n");
-    have_flags = TRUE;
-  }
-  if (GST_FLAG_IS_SET (element, GST_ELEMENT_THREAD_SUGGESTED)) {
-    n_print ("  GST_ELEMENT_THREADSUGGESTED\n");
-    have_flags = TRUE;
-  }
-  if (GST_FLAG_IS_SET (element, GST_ELEMENT_EVENT_AWARE)) {
-    n_print ("  GST_ELEMENT_EVENT_AWARE\n");
-    have_flags = TRUE;
-  }
   if (!have_flags)
     n_print ("  no flags set\n");
 
   if (GST_IS_BIN (element)) {
     n_print ("\n");
     n_print ("Bin Flags:\n");
-    if (GST_FLAG_IS_SET (element, GST_BIN_FLAG_MANAGER)) {
-      n_print ("  GST_BIN_FLAG_MANAGER\n");
-      have_flags = TRUE;
-    }
-    if (GST_FLAG_IS_SET (element, GST_BIN_SELF_SCHEDULABLE)) {
-      n_print ("  GST_BIN_SELF_SCHEDULABLE\n");
-      have_flags = TRUE;
-    }
-    if (GST_FLAG_IS_SET (element, GST_BIN_FLAG_PREFER_COTHREADS)) {
-      n_print ("  GST_BIN_FLAG_PREFER_COTHREADS\n");
-      have_flags = TRUE;
-    }
     if (!have_flags)
       n_print ("  no flags set\n");
   }
@@ -589,11 +606,7 @@ print_implementation_info (GstElement * element)
   n_print ("\n");
   n_print ("Element Implementation:\n");
 
-  if (element->loopfunc)
-    n_print ("  loopfunc()-based element: %s\n",
-        GST_DEBUG_FUNCPTR_NAME (element->loopfunc));
-  else
-    n_print ("  No loopfunc(), must be chain-based or not configured yet\n");
+  n_print ("  No loopfunc(), must be chain-based or not configured yet\n");
 
   n_print ("  Has change_state() function: %s\n",
       GST_DEBUG_FUNCPTR_NAME (gstelement_class->change_state));
@@ -658,7 +671,6 @@ print_pad_info (GstElement * element)
 {
   const GList *pads;
   GstPad *pad;
-  GstRealPad *realpad;
 
   n_print ("\n");
   n_print ("Pads:\n");
@@ -668,128 +680,58 @@ print_pad_info (GstElement * element)
     return;
   }
 
-  pads = gst_element_get_pad_list (element);
+  pads = element->pads;
   while (pads) {
     pad = GST_PAD (pads->data);
     pads = g_list_next (pads);
-    realpad = GST_PAD_REALIZE (pad);
 
     n_print ("");
 
-    if (gst_pad_get_direction (GST_PAD (realpad)) == GST_PAD_SRC)
+    if (gst_pad_get_direction (pad) == GST_PAD_SRC)
       g_print ("  SRC: '%s'", gst_pad_get_name (pad));
-    else if (gst_pad_get_direction (GST_PAD (realpad)) == GST_PAD_SINK)
+    else if (gst_pad_get_direction (pad) == GST_PAD_SINK)
       g_print ("  SINK: '%s'", gst_pad_get_name (pad));
     else
       g_print ("  UNKNOWN!!!: '%s'", gst_pad_get_name (pad));
 
-    if (GST_IS_GHOST_PAD (pad))
-      g_print (", ghost of real pad %s:%s\n", GST_DEBUG_PAD_NAME (realpad));
-    else
-      g_print ("\n");
+    g_print ("\n");
 
     n_print ("    Implementation:\n");
-    if (realpad->chainfunc)
+    if (pad->chainfunc)
       n_print ("      Has chainfunc(): %s\n",
-          GST_DEBUG_FUNCPTR_NAME (realpad->chainfunc));
-    if (realpad->getfunc)
-      n_print ("      Has getfunc(): %s\n",
-          GST_DEBUG_FUNCPTR_NAME (realpad->getfunc));
-    if (realpad->formatsfunc != gst_pad_get_formats_default) {
-      n_print ("      Supports seeking/conversion/query formats:\n");
-      print_formats (gst_pad_get_formats (GST_PAD (realpad)));
-    }
-    if (realpad->convertfunc != gst_pad_convert_default)
-      n_print ("      Has custom convertfunc(): %s\n",
-          GST_DEBUG_FUNCPTR_NAME (realpad->convertfunc));
-    if (realpad->eventfunc != gst_pad_event_default)
+          GST_DEBUG_FUNCPTR_NAME (pad->chainfunc));
+    if (pad->getrangefunc)
+      n_print ("      Has getrangefunc(): %s\n",
+          GST_DEBUG_FUNCPTR_NAME (pad->getrangefunc));
+    if (pad->eventfunc != gst_pad_event_default)
       n_print ("      Has custom eventfunc(): %s\n",
-          GST_DEBUG_FUNCPTR_NAME (realpad->eventfunc));
-    if (realpad->eventmaskfunc != gst_pad_get_event_masks_default) {
-      n_print ("        Provides event masks:\n");
-      print_event_masks (gst_pad_get_event_masks (GST_PAD (realpad)));
-    }
-    if (realpad->queryfunc != gst_pad_query_default)
+          GST_DEBUG_FUNCPTR_NAME (pad->eventfunc));
+    if (pad->queryfunc != gst_pad_query_default)
       n_print ("      Has custom queryfunc(): %s\n",
-          GST_DEBUG_FUNCPTR_NAME (realpad->queryfunc));
-    if (realpad->querytypefunc != gst_pad_get_query_types_default) {
+          GST_DEBUG_FUNCPTR_NAME (pad->queryfunc));
+    if (pad->querytypefunc != gst_pad_get_query_types_default) {
       n_print ("        Provides query types:\n");
-      print_query_types (gst_pad_get_query_types (GST_PAD (realpad)));
+      print_query_types (gst_pad_get_query_types (pad));
     }
 
-    if (realpad->intlinkfunc != gst_pad_get_internal_links_default)
+    if (pad->intlinkfunc != gst_pad_get_internal_links_default)
       n_print ("      Has custom intconnfunc(): %s\n",
-          GST_DEBUG_FUNCPTR_NAME (realpad->intlinkfunc));
+          GST_DEBUG_FUNCPTR_NAME (pad->intlinkfunc));
 
-    if (realpad->bufferallocfunc)
+    if (pad->bufferallocfunc)
       n_print ("      Has bufferallocfunc(): %s\n",
-          GST_DEBUG_FUNCPTR_NAME (realpad->bufferallocfunc));
+          GST_DEBUG_FUNCPTR_NAME (pad->bufferallocfunc));
 
     if (pad->padtemplate)
       n_print ("    Pad Template: '%s'\n", pad->padtemplate->name_template);
 
-    if (realpad->caps) {
+    if (pad->caps) {
       n_print ("    Capabilities:\n");
-      print_caps (realpad->caps, "      ");
+      print_caps (pad->caps, "      ");
     }
   }
 }
 
-static void
-print_dynamic_parameters_info (GstElement * element)
-{
-  GstDParamManager *dpman;
-  GParamSpec **specs = NULL;
-  gint x;
-
-  if ((dpman = gst_dpman_get_manager (element))) {
-    specs = gst_dpman_list_dparam_specs (dpman);
-  }
-
-  if (specs && specs[0] != NULL) {
-    n_print ("\n");
-    n_print ("Dynamic Parameters:\n");
-
-    for (x = 0; specs[x] != NULL; x++) {
-      g_print ("  %-20.20s: ", g_param_spec_get_name (specs[x]));
-
-      switch (G_PARAM_SPEC_VALUE_TYPE (specs[x])) {
-        case G_TYPE_INT64:
-          g_print ("64 Bit Integer (Default %" G_GINT64_FORMAT ", Range %"
-              G_GINT64_FORMAT " -> %" G_GINT64_FORMAT ")",
-              ((GParamSpecInt64 *) specs[x])->default_value,
-              ((GParamSpecInt64 *) specs[x])->minimum,
-              ((GParamSpecInt64 *) specs[x])->maximum);
-          break;
-        case G_TYPE_INT:
-          g_print ("Integer (Default %d, Range %d -> %d)",
-              ((GParamSpecInt *) specs[x])->default_value,
-              ((GParamSpecInt *) specs[x])->minimum,
-              ((GParamSpecInt *) specs[x])->maximum);
-          break;
-        case G_TYPE_FLOAT:
-          g_print ("Float. Default: %-8.8s %15.7g\n", "",
-              ((GParamSpecFloat *) specs[x])->default_value);
-          g_print ("%-23.23s Range: %15.7g - %15.7g", "",
-              ((GParamSpecFloat *) specs[x])->minimum,
-              ((GParamSpecFloat *) specs[x])->maximum);
-          break;
-        case G_TYPE_DOUBLE:
-          g_print ("Double. Default: %-8.8s %15.7g\n", "",
-              ((GParamSpecDouble *) specs[x])->default_value);
-          g_print ("%-23.23s Range: %15.7g - %15.7g", "",
-              ((GParamSpecDouble *) specs[x])->minimum,
-              ((GParamSpecDouble *) specs[x])->maximum);
-          break;
-        default:
-          g_print ("unknown %ld", G_PARAM_SPEC_VALUE_TYPE (specs[x]));
-      }
-      g_print ("\n");
-    }
-    g_free (specs);
-  }
-}
-
 #if 0
 static gint
 compare_signal_names (GSignalQuery * a, GSignalQuery * b)
@@ -858,6 +800,9 @@ print_signal_info (GstElement * element)
         if (G_TYPE_IS_FUNDAMENTAL (query->param_types[j])) {
           g_print (",\n%s%s%s arg%d", _name, indent,
               g_type_name (query->param_types[j]), j);
+        } else if (G_TYPE_IS_ENUM (query->param_types[j])) {
+          g_print (",\n%s%s%s arg%d", _name, indent,
+              g_type_name (query->param_types[j]), j);
         } else {
           g_print (",\n%s%s%s* arg%d", _name, indent,
               g_type_name (query->param_types[j]), j);
@@ -887,7 +832,7 @@ print_children_info (GstElement * element)
   if (!GST_IS_BIN (element))
     return;
 
-  children = (GList *) gst_bin_get_list (GST_BIN (element));
+  children = (GList *) GST_BIN (element)->children;
   if (children) {
     n_print ("\n");
     g_print ("Children:\n");
@@ -902,21 +847,26 @@ print_children_info (GstElement * element)
 static void
 print_element_list (gboolean print_all)
 {
-  GList *plugins;
+  int plugincount = 0, featurecount = 0;
+  GList *plugins, *orig_plugins;
 
-  plugins = gst_registry_pool_plugin_list ();
+  orig_plugins = plugins = gst_default_registry_get_plugin_list ();
   while (plugins) {
-    GList *features;
+    GList *features, *orig_features;
     GstPlugin *plugin;
 
     plugin = (GstPlugin *) (plugins->data);
     plugins = g_list_next (plugins);
+    plugincount++;
 
-    features = gst_plugin_get_feature_list (plugin);
+    orig_features = features =
+        gst_registry_get_feature_list_by_plugin (gst_registry_get_default (),
+        plugin->desc.name);
     while (features) {
       GstPluginFeature *feature;
 
       feature = GST_PLUGIN_FEATURE (features->data);
+      featurecount++;
 
       if (GST_IS_ELEMENT_FACTORY (feature)) {
         GstElementFactory *factory;
@@ -959,13 +909,6 @@ print_element_list (gboolean print_all)
           if (!print_all)
             g_print ("no extensions\n");
         }
-      } else if (GST_IS_SCHEDULER_FACTORY (feature)) {
-        GstSchedulerFactory *factory;
-
-        factory = GST_SCHEDULER_FACTORY (feature);
-        if (!print_all)
-          g_print ("%s:  %s: %s\n", plugin->desc.name,
-              GST_PLUGIN_FEATURE_NAME (factory), factory->longdesc);
       } else {
         if (!print_all)
           n_print ("%s:  %s (%s)\n", plugin->desc.name,
@@ -975,20 +918,33 @@ print_element_list (gboolean print_all)
 
       features = g_list_next (features);
     }
+
+    gst_plugin_feature_list_free (orig_features);
   }
+
+  gst_plugin_list_free (orig_plugins);
+
+  g_print ("\n");
+  g_print (_("Total count: "));
+  g_print (ngettext ("%d plugin", "%d plugins", plugincount), plugincount);
+  g_print (", ");
+  g_print (ngettext ("%d feature", "%d features", featurecount), featurecount);
+  g_print ("\n");
 }
 
 static void
 print_plugin_info (GstPlugin * plugin)
 {
   n_print ("Plugin Details:\n");
-  n_print ("  Name:\t\t%s\n", plugin->desc.name);
-  n_print ("  Description:\t%s\n", plugin->desc.description);
-  n_print ("  Filename:\t%s\n", plugin->filename ? plugin->filename : "(null)");
-  n_print ("  Version:\t%s\n", plugin->desc.version);
-  n_print ("  License:\t%s\n", plugin->desc.license);
-  n_print ("  Package:\t%s\n", plugin->desc.package);
-  n_print ("  Origin URL:\t%s\n", plugin->desc.origin);
+  n_print ("  Name:\t\t\t%s\n", plugin->desc.name);
+  n_print ("  Description:\t\t%s\n", plugin->desc.description);
+  n_print ("  Filename:\t\t%s\n",
+      plugin->filename ? plugin->filename : "(null)");
+  n_print ("  Version:\t\t%s\n", plugin->desc.version);
+  n_print ("  License:\t\t%s\n", plugin->desc.license);
+  n_print ("  Source module:\t%s\n", plugin->desc.source);
+  n_print ("  Binary package:\t%s\n", plugin->desc.package);
+  n_print ("  Origin URL:\t\t%s\n", plugin->desc.origin);
   n_print ("\n");
 }
 
@@ -999,11 +955,12 @@ print_plugin_features (GstPlugin * plugin)
   gint num_features = 0;
   gint num_elements = 0;
   gint num_types = 0;
-  gint num_schedulers = 0;
   gint num_indexes = 0;
   gint num_other = 0;
 
-  features = gst_plugin_get_feature_list (plugin);
+  features =
+      gst_registry_get_feature_list_by_plugin (gst_registry_get_default (),
+      plugin->desc.name);
 
   while (features) {
     GstPluginFeature *feature;
@@ -1014,7 +971,7 @@ print_plugin_features (GstPlugin * plugin)
       GstElementFactory *factory;
 
       factory = GST_ELEMENT_FACTORY (feature);
-      n_print ("  %s: %s\n", GST_OBJECT_NAME (factory),
+      n_print ("  %s: %s\n", GST_PLUGIN_FEATURE_NAME (factory),
           factory->details.longname);
       num_elements++;
     }
@@ -1044,12 +1001,6 @@ print_plugin_features (GstPlugin * plugin)
         g_print ("%s type: N/A\n", plugin->desc.name);
 
       num_types++;
-    } else if (GST_IS_SCHEDULER_FACTORY (feature)) {
-      GstSchedulerFactory *factory;
-
-      factory = GST_SCHEDULER_FACTORY (feature);
-      n_print ("  %s: %s\n", GST_OBJECT_NAME (factory), factory->longdesc);
-      num_schedulers++;
     } else {
       n_print ("  %s (%s)\n", gst_object_get_name (GST_OBJECT (feature)),
           g_type_name (G_OBJECT_TYPE (feature)));
@@ -1064,8 +1015,6 @@ print_plugin_features (GstPlugin * plugin)
     n_print ("  +-- %d elements\n", num_elements);
   if (num_types > 0)
     n_print ("  +-- %d types\n", num_types);
-  if (num_schedulers > 0)
-    n_print ("  +-- %d schedulers\n", num_schedulers);
   if (num_indexes > 0)
     n_print ("  +-- %d indexes\n", num_indexes);
   if (num_other > 0)
@@ -1080,33 +1029,20 @@ print_element_features (const gchar * element_name)
   GstPluginFeature *feature;
 
   /* FIXME implement other pretty print function for these */
-  feature = gst_registry_pool_find_feature (element_name,
-      GST_TYPE_SCHEDULER_FACTORY);
-  if (feature) {
-    n_print ("%s: a scheduler\n", element_name);
-    return 0;
-  }
 #ifndef GST_DISABLE_INDEX
-  feature = gst_registry_pool_find_feature (element_name,
+  feature = gst_default_registry_find_feature (element_name,
       GST_TYPE_INDEX_FACTORY);
   if (feature) {
     n_print ("%s: an index\n", element_name);
     return 0;
   }
 #endif
-  feature = gst_registry_pool_find_feature (element_name,
+  feature = gst_default_registry_find_feature (element_name,
       GST_TYPE_TYPE_FIND_FACTORY);
   if (feature) {
     n_print ("%s: a typefind function\n", element_name);
     return 0;
   }
-#ifndef GST_DISABLE_URI
-  feature = gst_registry_pool_find_feature (element_name, GST_TYPE_URI_HANDLER);
-  if (feature) {
-    n_print ("%s: an uri handler\n", element_name);
-    return 0;
-  }
-#endif
 
   return -1;
 }
@@ -1117,6 +1053,15 @@ print_element_info (GstElementFactory * factory, gboolean print_names)
   GstElement *element;
   gint maxlevel = 0;
 
+  factory =
+      GST_ELEMENT_FACTORY (gst_plugin_feature_load (GST_PLUGIN_FEATURE
+          (factory)));
+
+  if (!factory) {
+    g_print ("element plugin couldn't be loaded\n");
+    return -1;
+  }
+
   element = gst_element_factory_create (factory, NULL);
   if (!element) {
     g_print ("couldn't construct element for some reason\n");
@@ -1129,10 +1074,14 @@ print_element_info (GstElementFactory * factory, gboolean print_names)
     _name = "";
 
   print_factory_details_info (factory);
-  if (GST_PLUGIN_FEATURE (factory)->manager) {
-    GstPlugin *plugin = (GstPlugin *) GST_PLUGIN_FEATURE (factory)->manager;
+  if (GST_PLUGIN_FEATURE (factory)->plugin_name) {
+    GstPlugin *plugin;
 
-    print_plugin_info (plugin);
+    plugin = gst_registry_find_plugin (gst_registry_get_default (),
+        GST_PLUGIN_FEATURE (factory)->plugin_name);
+    if (plugin) {
+      print_plugin_info (plugin);
+    }
   }
 
   print_hierarchy (G_OBJECT_TYPE (element), 0, &maxlevel);
@@ -1145,36 +1094,184 @@ print_element_info (GstElementFactory * factory, gboolean print_names)
   print_index_info (element);
   print_pad_info (element);
   print_element_properties_info (element);
-  print_dynamic_parameters_info (element);
   print_signal_info (element);
   print_children_info (element);
 
-  if (_name != "")
+  gst_object_unref (factory);
+  if (_name[0] != '\0')
     g_free (_name);
 
   return 0;
 }
 
+
+static void
+print_plugin_automatic_install_info_codecs (GstElementFactory * factory)
+{
+  GstPadDirection direction;
+  const gchar *type_name;
+  const gchar *klass;
+  const GList *static_templates, *l;
+  GstCaps *caps = NULL;
+  guint i, num;
+
+  klass = gst_element_factory_get_klass (factory);
+  g_return_if_fail (klass != NULL);
+
+  if (strstr (klass, "Codec/") == NULL)
+    return;
+
+  if (strstr (klass, "Demuxer") ||
+      strstr (klass, "Decoder") ||
+      strstr (klass, "Depay") || strstr (klass, "Parser")) {
+    type_name = "decoder";
+    direction = GST_PAD_SINK;
+  } else if (strstr (klass, "Muxer") ||
+      strstr (klass, "Encoder") || strstr (klass, "Pay")) {
+    type_name = "encoder";
+    direction = GST_PAD_SRC;
+  } else {
+    g_printerr ("Unhandled Codec klass: %s\n", klass);
+    return;
+  }
+
+  /* decoder/demuxer sink pads should always be static and there should only
+   * be one, the same applies to encoders/muxers and source pads */
+  static_templates = gst_element_factory_get_static_pad_templates (factory);
+  for (l = static_templates; l != NULL; l = l->next) {
+    GstStaticPadTemplate *tmpl = NULL;
+
+    tmpl = (GstStaticPadTemplate *) l->data;
+    if (tmpl->direction == direction) {
+      caps = gst_static_pad_template_get_caps (tmpl);
+      break;
+    }
+  }
+
+  if (caps == NULL) {
+    g_printerr ("Couldn't find static pad template for %s '%s'\n",
+        type_name, GST_PLUGIN_FEATURE_NAME (factory));
+    return;
+  }
+
+  caps = gst_caps_make_writable (caps);
+  num = gst_caps_get_size (caps);
+  for (i = 0; i < num; ++i) {
+    GstStructure *s;
+    gchar *s_str;
+
+    s = gst_caps_get_structure (caps, i);
+    /* remove fields that are almost always just MIN-MAX of some sort
+     * in order to make the caps look less messy */
+    gst_structure_remove_field (s, "pixel-aspect-ratio");
+    gst_structure_remove_field (s, "framerate");
+    gst_structure_remove_field (s, "channels");
+    gst_structure_remove_field (s, "width");
+    gst_structure_remove_field (s, "height");
+    gst_structure_remove_field (s, "rate");
+    gst_structure_remove_field (s, "depth");
+    gst_structure_remove_field (s, "clock-rate");
+    s_str = gst_structure_to_string (s);
+    g_print ("%s-%s\n", type_name, s_str);
+    g_free (s_str);
+  }
+  gst_caps_unref (caps);
+}
+
+static void
+print_plugin_automatic_install_info_protocols (GstElementFactory * factory)
+{
+  gchar **protocols, **p;
+
+  protocols = gst_element_factory_get_uri_protocols (factory);
+  if (protocols != NULL && *protocols != NULL) {
+    switch (gst_element_factory_get_uri_type (factory)) {
+      case GST_URI_SINK:
+        for (p = protocols; *p != NULL; ++p)
+          g_print ("urisink-%s\n", *p);
+        break;
+      case GST_URI_SRC:
+        for (p = protocols; *p != NULL; ++p)
+          g_print ("urisource-%s\n", *p);
+        break;
+      default:
+        break;
+    }
+    g_strfreev (protocols);
+  }
+}
+
+static void
+print_plugin_automatic_install_info (GstPlugin * plugin)
+{
+  const gchar *plugin_name;
+  GList *features, *l;
+
+  plugin_name = gst_plugin_get_name (plugin);
+
+  /* not interested in typefind factories, only element factories */
+  features = gst_registry_get_feature_list (gst_registry_get_default (),
+      GST_TYPE_ELEMENT_FACTORY);
+
+  for (l = features; l != NULL; l = l->next) {
+    GstPluginFeature *feature;
+
+    feature = GST_PLUGIN_FEATURE (l->data);
+
+    /* only interested in the ones that are in the plugin we just loaded */
+    if (g_str_equal (plugin_name, feature->plugin_name)) {
+      GstElementFactory *factory;
+
+      g_print ("element-%s\n", gst_plugin_feature_get_name (feature));
+
+      factory = GST_ELEMENT_FACTORY (feature);
+      print_plugin_automatic_install_info_protocols (factory);
+      print_plugin_automatic_install_info_codecs (factory);
+    }
+  }
+
+  g_list_foreach (features, (GFunc) gst_object_unref, NULL);
+  g_list_free (features);
+}
+
 int
 main (int argc, char *argv[])
 {
   gboolean print_all = FALSE;
-  struct poptOption options[] = {
-    GST_TOOLS_POPT_VERSION,
-    {"print-all", 'a', POPT_ARG_NONE | POPT_ARGFLAG_STRIP, &print_all, 0,
+  gboolean print_aii = FALSE;
+  GOptionEntry options[] = {
+    {"print-all", 'a', 0, G_OPTION_ARG_NONE, &print_all,
         N_("Print all elements"), NULL},
-    POPT_TABLEEND
+    {"print-plugin-auto-install-info", '\0', 0, G_OPTION_ARG_NONE, &print_aii,
+        N_("Print a machine-parsable list of features the specified plugin "
+              "provides.\n                                       "
+              "Useful in connection with external automatic plugin "
+              "installation mechanisms"), NULL},
+    GST_TOOLS_GOPTION_VERSION,
+    {NULL}
   };
+  GOptionContext *ctx;
+  GError *err = NULL;
 
-#ifdef GETTEXT_PACKAGE
+#ifdef ENABLE_NLS
   bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 #endif
 
-  gst_init_with_popt_table (&argc, &argv, options);
-  gst_tools_print_version ("gst-inspect-0.8");
-  gst_control_init (&argc, &argv);
+  if (!g_thread_supported ())
+    g_thread_init (NULL);
+
+  ctx = g_option_context_new ("[ELEMENT-NAME | PLUGIN-NAME]");
+  g_option_context_add_main_entries (ctx, options, GETTEXT_PACKAGE);
+  g_option_context_add_group (ctx, gst_init_get_option_group ());
+  if (!g_option_context_parse (ctx, &argc, &argv, &err)) {
+    g_print ("Error initializing: %s\n", err->message);
+    exit (1);
+  }
+  g_option_context_free (ctx);
+
+  gst_tools_print_version ("gst-inspect");
 
   if (print_all && argc > 2) {
     g_print ("-a requires no extra arguments\n");
@@ -1195,21 +1292,45 @@ main (int argc, char *argv[])
     /* if there's a factory, print out the info */
     if (factory) {
       retval = print_element_info (factory, print_all);
+      gst_object_unref (factory);
     } else {
       retval = print_element_features (arg);
     }
 
     /* otherwise check if it's a plugin */
     if (retval) {
-      plugin = gst_registry_pool_find_plugin (arg);
+      plugin = gst_default_registry_find_plugin (arg);
 
       /* if there is such a plugin, print out info */
       if (plugin) {
-        print_plugin_info (plugin);
-        print_plugin_features (plugin);
+        if (print_aii) {
+          print_plugin_automatic_install_info (plugin);
+        } else {
+          print_plugin_info (plugin);
+          print_plugin_features (plugin);
+        }
       } else {
-        g_print ("No such element or plugin '%s'\n", arg);
-        return -1;
+        GError *error = NULL;
+
+        if (g_file_test (arg, G_FILE_TEST_EXISTS)) {
+          plugin = gst_plugin_load_file (arg, &error);
+
+          if (plugin) {
+            if (print_aii) {
+              print_plugin_automatic_install_info (plugin);
+            } else {
+              print_plugin_info (plugin);
+              print_plugin_features (plugin);
+            }
+          } else {
+            g_print (_("Could not load plugin file: %s\n"), error->message);
+            g_error_free (error);
+            return -1;
+          }
+        } else {
+          g_print (_("No such element or plugin '%s'\n"), arg);
+          return -1;
+        }
       }
     }
   }
